// AppLauncher の JavaScript
var AppLauncher = {
	
	/**
	 * 外部アプリケーションを起動する関数
	 * 現在は、テスト用のアラートを表示するだけ.
	 */
	launch: function( evt ) {
		try {
			// 設定値の取得
			var path = AppLauncher.getPref("path");
			window.alert( "launching \"" + path + "\"..." );
			// cf.https://developer.mozilla.org/ja/Code_snippets/Running_applications
			// 実行可能ファイルに対する nsILocalFile を作成する
			var file = Components.classes["@mozilla.org/file/local;1"]
									.createInstance(Components.interfaces.nsILocalFile);
			file.initWithPath(path);
			if( ! file.exists() ) {
				throw new Error("The File \"" + path + "\" doesn't exist!");
			}
			// nsIProcess を作成する 
			var process = Components.classes["@mozilla.org/process/util;1"]
									.createInstance(Components.interfaces.nsIProcess);
			process.init(file);
			// プロセスの起動
			// 最初のパラメータが true なら、スレッドはプロセスが終わるまでブロックされる。
			// 2 つ目と 3 つ目のパラメータは、プロセスのコマンドライン引数
			// var args = ["argument1", "argument2"];
			var args = []; // とりあえず今は引数なし
			process.run(false, args, args.length);
		} catch(e) {
			window.alert("[EXCEPTION] An exception occured in \"AppLauncher\": \n" + e.message);
		}
	},
	
	/**
	 * Preference 情報を取得する関数 (設定の情報)
	 * @param prefString 取得するキー
	 */
	// cf.https://developer.mozilla.org/Ja/Code_snippets/Preferences
	getPref: function( prefString ) {
		var branchString = "extensions.applauncher.";
		// 設定の情報を取得する XPCOM オブジェクトの生成
		var prefSvc = Components.classes["@mozilla.org/preferences-service;1"]
								.getService(Components.interfaces.nsIPrefService);
		var prefBranch = prefSvc.getBranch(branchString);
		// タイプ別に取得する関数を分ける
		switch( prefBranch.getPrefType(prefString) ) {
			// 設定値が文字の場合
			case Components.interfaces.nsIPrefBranch.PREF_STRING:
				return prefBranch.getCharPref(prefString);
				break;
			// 設定値が数値の場合
			case Components.interfaces.nsIPrefBranch.PREF_INT:
				return prefBranch.getIntPref(prefString);
				break;
			// 設定値が真偽値の場合
			case Components.interfaces.nsIPrefBranch.PREF_BOOL:
				return prefBranch.getBoolPref(prefString);
				break;
		}
	},
	
	/**
	 * 初期化関数
	 * AppLauncher に関するコンテキストメニューの変更を行う.
	 */
	init: function() {
		// XUL で追加済みの "menuitem" 要素を取得し、イベントリスナを追加する
		var item = document.getElementById("info.vividcode.applauncher.contextmenu.item1");
		if( item ) {
			// イベントリスナの追加
			item.addEventListener("command", AppLauncher.launch, false);
		}
		// コンテキストメニュー内の、AppLauncher に関する "menupopup" 要素 (id で指定) を取得
		var menupopup = document.getElementById("info.vividcode.applauncher.contextmenu.items");
		if( menupopup ) {
			// "menuitem" 要素の作成
			item = document.createElementNS(
								"http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul",
								"menuitem" );
			// "label" 属性に "adding item!" という値を設定
			item.setAttribute("label", "item2");
			// イベントリスナの追加
			item.addEventListener("command", AppLauncher.launch, false);
			// "menupopup" 要素の子ノードに追加
			menupopup.appendChild(item);
		}
		menupopup = null;
		item = null;
	},
	
}

// 初期化関数をロード時に実行する
window.addEventListener("load", AppLauncher.init, false);
